@@ -74,6 +74,23 @@ def test_pull_blob(dxf_obj):
     assert ex.value.got == hashlib.sha256().hexdigest()
     assert ex.value.expected == pytest.blob1_hash
 
+def test_pull_and_push_blob(dxf_obj):
+    it = dxf_obj.pull_blob(pytest.blob1_hash)
+    state = {'total': 0}
+    sha256 = hashlib.sha256()
+    def progress(dgst, chunk):
+        assert dgst == pytest.blob1_hash
+        state['total'] += len(chunk)
+        sha256.update(chunk)
+    assert dxf_obj.push_blob(data=it,
+                             digest=pytest.blob1_hash,
+                             progress=progress,
+                             check_exists=False) == \
+           pytest.blob1_hash
+    assert state['total'] == pytest.blob1_size
+    assert sha256.hexdigest() == pytest.blob1_hash
+    _pull_blob(dxf_obj, pytest.blob1_hash, pytest.blob1_size, None)
+
 def test_set_alias(dxf_obj):
     dxf_obj.set_alias('hello', pytest.blob1_hash)
     dxf_obj.set_alias('there', pytest.blob1_hash, pytest.blob2_hash)
@@ -94,13 +111,21 @@ def test_context_manager(dxf_obj):
 def test_manifest(dxf_obj):
     manifest = dxf_obj.set_alias('mani_test', pytest.blob1_hash)
     assert manifest
+    if dxf_obj.regver != 2.2:
+        assert dxf_obj.get_manifest('mani_test') == manifest
+    #assert json.dumps(json.loads(dxf_obj.get_manifest('mani_test')),
+    #                  sort_keys=True) == \
+    #       json.dumps(json.loads(manifest), sort_keys=True)
     assert dxf_obj.get_alias(manifest=manifest) == [pytest.blob1_hash]
     if json.loads(manifest)['schemaVersion'] == 1:
         with pytest.raises(jws.exceptions.SignatureError):
             dxf_obj.get_alias(manifest=' '+manifest)
+    if dxf_obj.regver != 2.2:
+        dxf_obj.set_manifest('mani_test2', manifest)
+        assert dxf_obj.get_alias('mani_test2') == [pytest.blob1_hash]
 
 def test_unsigned_manifest_v1(dxf_obj):
-    manifest = dxf_obj.make_unsigned_manifest('mani_test2', pytest.blob2_hash)
+    manifest = dxf_obj.make_unsigned_manifest('mani_test3', pytest.blob2_hash)
     assert manifest
     with pytest.raises(KeyError):
         dxf_obj.get_alias(manifest=manifest)
